@@ -46,7 +46,15 @@ namespace CLI {
 #endif
 
 namespace detail {
-enum class Classifier { NONE, POSITIONAL_MARK, SHORT, LONG, WINDOWS_STYLE, SUBCOMMAND, SUBCOMMAND_TERMINATOR };
+enum class Classifier : std::uint8_t {
+    NONE,
+    POSITIONAL_MARK,
+    SHORT,
+    LONG,
+    WINDOWS_STYLE,
+    SUBCOMMAND,
+    SUBCOMMAND_TERMINATOR
+};
 struct AppFriend;
 }  // namespace detail
 
@@ -60,7 +68,7 @@ CLI11_INLINE std::string help(const App *app, const Error &e);
 
 /// enumeration of modes of how to deal with extras in config files
 
-enum class config_extras_mode : char { error = 0, ignore, ignore_all, capture };
+enum class config_extras_mode : std::uint8_t { error = 0, ignore, ignore_all, capture };
 
 class App;
 
@@ -242,7 +250,7 @@ class App {
     /// specify that positional arguments come at the end of the argument sequence not inheritable
     bool positionals_at_end_{false};
 
-    enum class startup_mode : char { stable, enabled, disabled };
+    enum class startup_mode : std::uint8_t { stable, enabled, disabled };
     /// specify the startup mode for the app
     /// stable=no change, enabled= startup enabled, disabled=startup disabled
     startup_mode default_startup{startup_mode::stable};
@@ -263,6 +271,9 @@ class App {
     /// indicator that the subcommand should allow non-standard option arguments, such as -single_dash_flag
     bool allow_non_standard_options_{false};
 
+    /// indicator to allow subcommands to match with prefix matching
+    bool allow_prefix_matching_{false};
+
     /// Counts the number of times this command/subcommand was parsed
     std::uint32_t parsed_{0U};
 
@@ -401,6 +412,11 @@ class App {
         return this;
     }
 
+    /// allow prefix matching for subcommands
+    App *allow_subcommand_prefix_matching(bool allowed = true) {
+        allow_prefix_matching_ = allowed;
+        return this;
+    }
     /// Set the subcommand to be disabled by default, so on clear(), at the start of each parse it is disabled
     App *disabled_by_default(bool disable = true) {
         if(disable) {
@@ -639,13 +655,15 @@ class App {
     Option *add_flag(std::string flag_name) { return _add_flag_internal(flag_name, CLI::callback_t(), std::string{}); }
 
     /// Add flag with description but with no variable assignment or callback
-    /// takes a constant string,  if a variable string is passed that variable will be assigned the results from the
-    /// flag
+    /// takes a constant string or a rvalue reference to a string,  if a variable string is passed that variable will be
+    /// assigned the results from the flag
     template <typename T,
-              enable_if_t<std::is_const<T>::value && std::is_constructible<std::string, T>::value, detail::enabler> =
-                  detail::dummy>
-    Option *add_flag(std::string flag_name, T &flag_description) {
-        return _add_flag_internal(flag_name, CLI::callback_t(), flag_description);
+              enable_if_t<(std::is_const<typename std::remove_reference<T>::type>::value ||
+                           std::is_rvalue_reference<T &&>::value) &&
+                              std::is_constructible<std::string, typename std::remove_reference<T>::type>::value,
+                          detail::enabler> = detail::dummy>
+    Option *add_flag(std::string flag_name, T &&flag_description) {
+        return _add_flag_internal(flag_name, CLI::callback_t(), std::forward<T>(flag_description));
     }
 
     /// Other type version accepts all other types that are not vectors such as bool, enum, string or other classes
@@ -724,7 +742,10 @@ class App {
         auto option_group = std::make_shared<T>(std::move(group_description), group_name, this);
         auto *ptr = option_group.get();
         // move to App_p for overload resolution on older gcc versions
-        App_p app_ptr = std::dynamic_pointer_cast<App>(option_group);
+        App_p app_ptr = std::static_pointer_cast<App>(option_group);
+        // don't inherit the footer in option groups and clear the help flag by default
+        app_ptr->footer_ = "";
+        app_ptr->set_help_flag();
         add_subcommand(std::move(app_ptr));
         return ptr;
     }
@@ -1155,9 +1176,12 @@ class App {
     /// Get the status of silence
     CLI11_NODISCARD bool get_silent() const { return silent_; }
 
-    /// Get the status of silence
+    /// Get the status of allowing non standard option names
     CLI11_NODISCARD bool get_allow_non_standard_option_names() const { return allow_non_standard_options_; }
 
+    /// Get the status of allowing prefix matching for subcommands
+    CLI11_NODISCARD bool get_allow_subcommand_prefix_matching() const { return allow_prefix_matching_; }
+
     /// Get the status of disabled
     CLI11_NODISCARD bool get_immediate_callback() const { return immediate_callback_; }
 
@@ -1216,9 +1240,18 @@ class App {
     /// Get a display name for an app
     CLI11_NODISCARD std::string get_display_name(bool with_aliases = false) const;
 
-    /// Check the name, case-insensitive and underscore insensitive if set
+    /// Check the name, case-insensitive and underscore insensitive, and prefix matching if set
+    /// @return true if matched
     CLI11_NODISCARD bool check_name(std::string name_to_check) const;
 
+    /// @brief  enumeration of matching possibilities
+    enum class NameMatch : std::uint8_t { none = 0, exact = 1, prefix = 2 };
+
+    /// Check the name, case-insensitive and underscore insensitive if set
+    /// @return NameMatch::none if no match, NameMatch::exact if the match is exact NameMatch::prefix if prefix is
+    /// enabled and a prefix matches
+    CLI11_NODISCARD NameMatch check_name_detail(std::string name_to_check) const;
+
     /// Get the groups available directly from this option (in order)
     CLI11_NODISCARD std::vector<std::string> get_groups() const;
 
@@ -1270,12 +1303,12 @@ class App {
     void _process_env();
 
     /// Process callbacks. Runs on *all* subcommands.
-    void _process_callbacks();
+    void _process_callbacks(CallbackPriority priority);
 
     /// Run help flag processing if any are found.
     ///
     /// The flags allow recursive calls to remember if there was a help flag on a parent.
-    void _process_help_flags(bool trigger_help = false, bool trigger_all_help = false) const;
+    void _process_help_flags(CallbackPriority priority, bool trigger_help = false, bool trigger_all_help = false) const;
 
     /// Verify required options and cross requirements. Subcommands too (only if selected).
     void _process_requirements();
@@ -1311,6 +1344,9 @@ class App {
     /// Fill in a single config option
     bool _parse_single_config(const ConfigItem &item, std::size_t level = 0);
 
+    /// @brief store the results for a flag like option
+    bool _add_flag_like_result(Option *op, const ConfigItem &item, const std::vector<std::string> &inputs);
+
     /// Parse "one" argument (some may eat more than one), delegate to parent if fails, add to missing if missing
     /// from main return false if the parse has failed and needs to return to parent
     bool _parse_single(std::vector<std::string> &args, bool &positional_only);
